@@ -34,6 +34,13 @@
 #define trace_xfs_alloc_small_done(a)		((void) 0)
 #define trace_xfs_alloc_small_error(a)		((void) 0)
 #define trace_xfs_alloc_vextent_badargs(a)	((void) 0)
+#define trace_xfs_alloc_vextent_skip_deadlock(...)	((void) 0)
+#define trace_xfs_alloc_vextent_finish(...)	((void) 0)
+#define trace_xfs_alloc_vextent_this_ag(...)	((void) 0)
+#define trace_xfs_alloc_vextent_first_ag(...)	((void) 0)
+#define trace_xfs_alloc_vextent_start_ag(...)	((void) 0)
+#define trace_xfs_alloc_vextent_near_bno(...)	((void) 0)
+#define trace_xfs_alloc_vextent_exact_bno(...)	((void) 0)
 #define trace_xfs_alloc_vextent_nofix(a)	((void) 0)
 #define trace_xfs_alloc_vextent_noagbp(a)	((void) 0)
 #define trace_xfs_alloc_vextent_loopfailed(a)	((void) 0)
@@ -180,9 +187,12 @@
 
 /* set c = c to avoid unused var warnings */
 #define trace_xfs_write_extent(a,b,c,d)	((c) = (c))
-#define trace_xfs_perag_get(a,b,c,d)	((c) = (c))
-#define trace_xfs_perag_get_tag(a,b,c,d) ((c) = (c))
-#define trace_xfs_perag_put(a,b,c,d)	((c) = (c))
+#define trace_xfs_perag_get(c,d)		((c) = (c))
+#define trace_xfs_perag_get_tag(c,d)		((c) = (c))
+#define trace_xfs_perag_grab(...)		((void) 0)
+#define trace_xfs_perag_grab_tag(...)		((void) 0)
+#define trace_xfs_perag_put(c,d)		((c) = (c))
+#define trace_xfs_perag_rele(...)		((void) 0)
 
 #define trace_xfs_trans_alloc(a,b)		((void) 0)
 #define trace_xfs_trans_cancel(a,b)		((void) 0)
@@ -322,9 +332,4 @@
 
 #define trace_xfs_fs_mark_healthy(a,b)		((void) 0)
 
-/* set c = c to avoid unused var warnings */
-#define trace_xfs_perag_get(a,b,c,d)		((c) = (c))
-#define trace_xfs_perag_get_tag(a,b,c,d)	((c) = (c))
-#define trace_xfs_perag_put(a,b,c,d)		((c) = (c))
-
 #endif /* __TRACE_H__ */
